/**

**/
#include <stdint.h>

#include "nrf_delay.h"
#include "app_error.h"

#include "nrf_drv_ppi.h"
#include "nrf_drv_timer.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#define PPI_COUNT_TIMER1_COUNT		(10)									/* timer1 count 10 (10us * 10 = 100us) */
#define PPI_COUNT_TIMER2_INTERVAL	(10)									/* timer2 interval in 10us */

static const nrf_drv_timer_t t1_counter = NRF_DRV_TIMER_INSTANCE(1);		/* timer1 counter mode */
static const nrf_drv_timer_t t2_cycle = NRF_DRV_TIMER_INSTANCE(2);			/* timer2 100us */

static nrf_ppi_channel_t m_ppi_channel2;

static uint16_t Control_counter = 0;								/* Control counter  [0-65535(0-6553.5ms)] (1count:100us) */


/*-----------------------------------------*/
/*--< Program operation processing area >--*/
/*-----------------------------------------*/

/*-< Timer event handler. Not used since Timer1 and Timer2 are used only for PPI. >-*/
static void empty_timer_handler(nrf_timer_event_t event_type, void * p_context)
{
}

void timer1_event_handler(nrf_timer_event_t event_type, void* p_context)
{
            Control_counter ++;				/* Control_counter update */
}

/*-< Set timer1:CC1 to counter mode		(timer 1 initialization)	>-*/
static void timer1_counter_init(void)
{
	nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
	timer_cfg.mode = NRF_TIMER_MODE_COUNTER;		/* NRF_TIMER_MODE_TIMER -> NRF_TIMER_MODE_COUNTER	*/

	ret_code_t err_code = nrf_drv_timer_init(&t1_counter, &timer_cfg, timer1_event_handler);
	APP_ERROR_CHECK(err_code);

//	nrfx_timer_capture(&t1_counter, NRF_TIMER_CC_CHANNEL1);
    nrf_drv_timer_extended_compare(&t1_counter,
									NRF_TIMER_CC_CHANNEL1,
									COUNT_TIMER1_COUNT,
									NRF_TIMER_SHORT_COMPARE1_CLEAR_MASK, true);
}

/*-< Set timer2:CC2 to 100us timer mode		(timer 2 initialization)	>-*/
static void timer2_cycle_init(void)
{
    nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
	
	ret_code_t err_code = nrf_drv_timer_init(&t2_cycle, &timer_cfg, empty_timer_handler);
    APP_ERROR_CHECK(err_code);

    nrf_drv_timer_extended_compare(&t2_cycle,
									NRF_TIMER_CC_CHANNEL2,
									nrf_drv_timer_us_to_ticks(&t2_cycle, PPI_COUNT_TIMER2_INTERVAL),
									NRF_TIMER_SHORT_COMPARE2_CLEAR_MASK, false);
}

/*-< PPI controls the process of counting at 100us	(PPI 100us(timer1) counter(timer2) initialization)	>-*/
static void ppi_Sync_counter_init(void)
{
	uint32_t err_code = NRF_SUCCESS;

	err_code = nrf_drv_ppi_init();
	APP_ERROR_CHECK(err_code);

	err_code = nrf_drv_ppi_channel_alloc(&m_ppi_channel2);
	APP_ERROR_CHECK(err_code);

	err_code = nrf_drv_ppi_channel_assign(m_ppi_channel2,
                                          nrf_drv_timer_event_address_get(&t2_cycle, NRF_TIMER_EVENT_COMPARE2),
                                          nrf_drv_timer_task_address_get(&t1_counter,  NRF_TIMER_TASK_COUNT));
	APP_ERROR_CHECK(err_code);

	err_code = nrf_drv_ppi_channel_enable(m_ppi_channel2);
	APP_ERROR_CHECK(err_code);
}

/*--------------------*/
/*-< Main process	>-*/
/*--------------------*/
int main(void)
{
	/* Initialize */
	ppi_Sync_counter_init();	
	timer1_counter_init();
	timer2_cycle_init();	
	
	/* Start */
	nrf_drv_timer_enable(&t1_counter);
	nrf_drv_timer_enable(&t2_cycle);
	
	while (true)
	{
		/**/
	}
}

/** @} */
